/**
 * data-structure-typed
 *
 * @author Tyler Zeng
 * @copyright Copyright (c) 2022 Tyler Zeng <zrwusa@gmail.com>
 * @license MIT License
 */
import type { BFSCallback, BinaryTreeNodeKey, BinaryTreeNodeNested, BinaryTreeOptions, MapCallback } from '../../types';
import { BinaryTreeDeletedResult, DefaultMapCallback, DFSOrderPattern, FamilyPosition, IterationType } from '../../types';
import { IBinaryTree } from '../../interfaces';
/**
 * Represents a node in a binary tree.
 * @template V - The type of data stored in the node.
 * @template N - The type of the family relationship in the binary tree.
 */
export declare class BinaryTreeNode<V = any, N extends BinaryTreeNode<V, N> = BinaryTreeNode<V, BinaryTreeNodeNested<V>>> {
    /**
     * The key associated with the node.
     */
    key: BinaryTreeNodeKey;
    /**
     * The value stored in the node.
     */
    val: V | undefined;
    /**
     * The parent node of the current node.
     */
    parent: N | null | undefined;
    /**
     * Creates a new instance of BinaryTreeNode.
     * @param {BinaryTreeNodeKey} key - The key associated with the node.
     * @param {V} val - The value stored in the node.
     */
    constructor(key: BinaryTreeNodeKey, val?: V);
    private _left;
    /**
     * Get the left child node.
     */
    get left(): N | null | undefined;
    /**
     * Set the left child node.
     * @param {N | null | undefined} v - The left child node.
     */
    set left(v: N | null | undefined);
    private _right;
    /**
     * Get the right child node.
     */
    get right(): N | null | undefined;
    /**
     * Set the right child node.
     * @param {N | null | undefined} v - The right child node.
     */
    set right(v: N | null | undefined);
    /**
     * Get the position of the node within its family.
     * @returns {FamilyPosition} - The family position of the node.
     */
    get familyPosition(): FamilyPosition;
}
/**
 * Represents a binary tree data structure.
 * @template N - The type of the binary tree's nodes.
 */
export declare class BinaryTree<V = any, N extends BinaryTreeNode<V, N> = BinaryTreeNode<V, BinaryTreeNodeNested<V>>> implements IBinaryTree<V, N> {
    /**
     * Creates a new instance of BinaryTree.
     * @param {BinaryTreeOptions} [options] - The options for the binary tree.
     */
    constructor(options?: BinaryTreeOptions);
    private _iterationType;
    /**
     * Get the iteration type used in the binary tree.
     */
    get iterationType(): IterationType;
    /**
     * Set the iteration type for the binary tree.
     * @param {IterationType} v - The new iteration type to set.
     */
    set iterationType(v: IterationType);
    private _root;
    /**
     * Get the root node of the binary tree.
     */
    get root(): N | null;
    private _size;
    /**
     * Get the number of nodes in the binary tree.
     */
    get size(): number;
    /**
     * Creates a new instance of BinaryTreeNode with the given key and value.
     * @param {BinaryTreeNodeKey} key - The key for the new node.
     * @param {V} val - The value for the new node.
     * @returns {N} - The newly created BinaryTreeNode.
     */
    createNode(key: BinaryTreeNodeKey, val?: V): N;
    /**
     * Clear the binary tree, removing all nodes.
     */
    clear(): void;
    /**
     * Check if the binary tree is empty.
     * @returns {boolean} - True if the binary tree is empty, false otherwise.
     */
    isEmpty(): boolean;
    /**
     * Add a node with the given key and value to the binary tree.
     * @param {BinaryTreeNodeKey | N | null} keyOrNode - The key or node to add to the binary tree.
     * @param {V} val - The value for the new node (optional).
     * @returns {N | null | undefined} - The inserted node, or null if nothing was inserted, or undefined if the operation failed.
     */
    add(keyOrNode: BinaryTreeNodeKey | N | null, val?: V): N | null | undefined;
    /**
     * The `addMany` function takes an array of binary tree node IDs or nodes, and optionally an array of corresponding data
     * values, and adds them to the binary tree.
     * @param {(BinaryTreeNodeKey | null)[] | (N | null)[]} keysOrNodes - An array of BinaryTreeNodeKey or BinaryTreeNode
     * objects, or null values.
     * @param {V[]} [values] - The `values` parameter is an optional array of values (`V[]`) that corresponds to
     * the nodes or node IDs being added. It is used to set the value of each node being added. If `values` is not provided,
     * the value of the nodes will be `undefined`.
     * @returns The function `addMany` returns an array of `N`, `null`, or `undefined` values.
     */
    addMany(keysOrNodes: (BinaryTreeNodeKey | null)[] | (N | null)[], values?: V[]): (N | null | undefined)[];
    /**
     * The `refill` function clears the binary tree and adds multiple nodes with the given IDs or nodes and optional data.
     * @param {(BinaryTreeNodeKey | N)[]} keysOrNodes - The `keysOrNodes` parameter is an array that can contain either
     * `BinaryTreeNodeKey` or `N` values.
     * @param {N[] | Array<V>} [data] - The `data` parameter is an optional array of values that will be assigned to
     * the nodes being added. If provided, the length of the `data` array should be equal to the length of the `keysOrNodes`
     * array. Each value in the `data` array will be assigned to the
     * @returns The method is returning a boolean value.
     */
    refill(keysOrNodes: (BinaryTreeNodeKey | null)[] | (N | null)[], data?: Array<V>): boolean;
    delete<C extends MapCallback<N>>(identifier: ReturnType<C> | N): BinaryTreeDeletedResult<N>[];
    delete<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C): BinaryTreeDeletedResult<N>[];
    /**
     * The function `getDepth` calculates the depth of a given node in a binary tree relative to a
     * specified root node.
     * @param {BinaryTreeNodeKey | N | null} distNode - The `distNode` parameter represents the node
     * whose depth we want to find in the binary tree. It can be either a node object (`N`), a key value
     * of the node (`BinaryTreeNodeKey`), or `null`.
     * @param {BinaryTreeNodeKey | N | null} beginRoot - The `beginRoot` parameter represents the
     * starting node from which we want to calculate the depth. It can be either a node object or the key
     * of a node in the binary tree. If no value is provided for `beginRoot`, it defaults to the root
     * node of the binary tree.
     * @returns the depth of the `distNode` relative to the `beginRoot`.
     */
    getDepth(distNode: BinaryTreeNodeKey | N | null, beginRoot?: BinaryTreeNodeKey | N | null): number;
    /**
     * The `getHeight` function calculates the maximum height of a binary tree using either recursive or
     * iterative approach.
     * @param {BinaryTreeNodeKey | N | null} beginRoot - The `beginRoot` parameter represents the
     * starting node from which the height of the binary tree is calculated. It can be either a node
     * object (`N`), a key value of a node in the tree (`BinaryTreeNodeKey`), or `null` if no starting
     * node is specified. If `
     * @param iterationType - The `iterationType` parameter is used to determine whether to calculate the
     * height of the binary tree using a recursive approach or an iterative approach. It can have two
     * possible values:
     * @returns the height of the binary tree.
     */
    getHeight(beginRoot?: BinaryTreeNodeKey | N | null, iterationType?: IterationType): number;
    /**
     * The `getMinHeight` function calculates the minimum height of a binary tree using either a
     * recursive or iterative approach.
     * @param {N | null} beginRoot - The `beginRoot` parameter is the starting node from which we want to
     * calculate the minimum height of the tree. It is optional and defaults to the root of the tree if
     * not provided.
     * @param iterationType - The `iterationType` parameter is used to determine the method of iteration
     * to calculate the minimum height of a binary tree. It can have two possible values:
     * @returns The function `getMinHeight` returns the minimum height of a binary tree.
     */
    getMinHeight(beginRoot?: N | null, iterationType?: IterationType): number;
    /**
     * The function checks if a binary tree is perfectly balanced by comparing the minimum height and the
     * height of the tree.
     * @param {N | null} beginRoot - The parameter `beginRoot` is of type `N | null`, which means it can
     * either be of type `N` (representing a node in a tree) or `null` (representing an empty tree).
     * @returns The method is returning a boolean value.
     */
    isPerfectlyBalanced(beginRoot?: N | null): boolean;
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N): N[];
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C): N[];
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N, onlyOne: boolean): N[];
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, onlyOne: boolean): N[];
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, onlyOne: boolean, beginRoot: N | null): N[];
    getNodes<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, onlyOne: boolean, beginRoot: N | null, iterationType: IterationType): N[];
    has<C extends MapCallback<N>>(identifier: ReturnType<C> | N): boolean;
    has<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C): boolean;
    has<C extends MapCallback<N>>(identifier: ReturnType<C> | N, beginRoot: N | null): boolean;
    has<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, beginRoot: N | null): boolean;
    get<C extends MapCallback<N>>(identifier: ReturnType<C> | N): N | null;
    get<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C): N | null;
    get<C extends MapCallback<N>>(identifier: ReturnType<C> | N, beginRoot: N | null): N | null;
    get<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, beginRoot: N | null): N | null;
    get<C extends MapCallback<N>>(identifier: ReturnType<C> | N, callback: C, beginRoot: N | null, iterationType: IterationType): N | null;
    /**
     * The function `getPathToRoot` returns an array of nodes starting from a given node and traversing
     * up to the root node, with the option to reverse the order of the nodes.
     * @param {N} beginRoot - The `beginRoot` parameter represents the starting node from which you want
     * to find the path to the root node.
     * @param [isReverse=true] - The `isReverse` parameter is a boolean flag that determines whether the
     * resulting path should be reversed or not. If `isReverse` is set to `true`, the path will be
     * reversed before returning it. If `isReverse` is set to `false` or not provided, the path will
     * @returns The function `getPathToRoot` returns an array of type `N[]`.
     */
    getPathToRoot(beginRoot: N, isReverse?: boolean): N[];
    /**
     * The function `getLeftMost` returns the leftmost node in a binary tree, either using recursive or
     * iterative traversal.
     * @param {BinaryTreeNodeKey | N | null} beginRoot - The `beginRoot` parameter is the starting point
     * for finding the leftmost node in a binary tree. It can be either a node object (`N`), a key value
     * of a node (`BinaryTreeNodeKey`), or `null` if the tree is empty.
     * @param iterationType - The `iterationType` parameter is used to determine the type of iteration to
     * be performed when finding the leftmost node in a binary tree. It can have two possible values:
     * @returns The function `getLeftMost` returns the leftmost node (`N`) in a binary tree. If there is
     * no leftmost node, it returns `null`.
     */
    getLeftMost(beginRoot?: BinaryTreeNodeKey | N | null, iterationType?: IterationType): N | null;
    /**
     * The function `getRightMost` returns the rightmost node in a binary tree, either recursively or
     * iteratively.
     * @param {N | null} beginRoot - The `beginRoot` parameter is the starting node from which we want to
     * find the rightmost node. It is of type `N | null`, which means it can either be a node of type `N`
     * or `null`. If it is `null`, it means there is no starting node
     * @param iterationType - The `iterationType` parameter is used to determine the type of iteration to
     * be performed when finding the rightmost node in a binary tree. It can have two possible values:
     * @returns The function `getRightMost` returns the rightmost node (`N`) in a binary tree. If the
     * `beginRoot` parameter is `null`, it returns `null`.
     */
    getRightMost(beginRoot?: N | null, iterationType?: IterationType): N | null;
    /**
     * The function `isSubtreeBST` checks if a given binary tree is a valid binary search tree.
     * @param {N} beginRoot - The `beginRoot` parameter is the root node of the binary tree that you want
     * to check if it is a binary search tree (BST) subtree.
     * @param iterationType - The `iterationType` parameter is an optional parameter that specifies the
     * type of iteration to use when checking if a subtree is a binary search tree (BST). It can have two
     * possible values:
     * @returns The function `isSubtreeBST` returns a boolean value.
     */
    isSubtreeBST(beginRoot: N, iterationType?: IterationType): boolean;
    /**
     * The function checks if a binary tree is a binary search tree.
     * @param iterationType - The parameter "iterationType" is used to specify the type of iteration to
     * be used when checking if the binary tree is a binary search tree (BST). It is an optional
     * parameter with a default value of "this.iterationType". The value of "this.iterationType" is not
     * provided in
     * @returns a boolean value.
     */
    isBST(iterationType?: IterationType): boolean;
    /**
     * The function `subTreeTraverse` traverses a binary tree and applies a callback function to each
     * node, either recursively or iteratively.
     * @param callback - The `callback` parameter is a function that will be called on each node in the
     * subtree traversal. It takes a single argument, which is the current node being traversed, and
     * returns a value. The return values from each callback invocation will be collected and returned as
     * an array.
     * @param {BinaryTreeNodeKey | N | null} beginRoot - The `beginRoot` parameter is the starting point
     * for traversing the subtree. It can be either a node object, a key value of a node, or `null` to
     * start from the root of the tree.
     * @param iterationType - The `iterationType` parameter determines the type of traversal to be
     * performed on the binary tree. It can have two possible values:
     * @returns The function `subTreeTraverse` returns an array of `MapCallbackReturn<N>`.
     */
    subTreeTraverse<C extends MapCallback<N>>(callback?: C, beginRoot?: BinaryTreeNodeKey | N | null, iterationType?: IterationType): ReturnType<C>[];
    /**
     * The `dfs` function performs a depth-first search traversal on a binary tree, executing a callback
     * function on each node according to a specified order pattern.
     * @param callback - The `callback` parameter is a function that will be called on each node during
     * the depth-first search traversal. It takes a node as input and returns a value. The default value
     * is `this._defaultCallbackByKey`, which is a callback function defined elsewhere in the code.
     * @param {DFSOrderPattern} [pattern=in] - The `pattern` parameter determines the order in which the
     * nodes are visited during the depth-first search. There are three possible values for `pattern`:
     * @param {N | null} beginRoot - The `beginRoot` parameter is the starting node for the depth-first
     * search. It determines where the search will begin in the tree or graph structure. If `beginRoot`
     * is `null`, an empty array will be returned.
     * @param {IterationType} iterationType - The `iterationType` parameter determines the type of
     * iteration used in the depth-first search algorithm. It can have two possible values:
     * @returns The function `dfs` returns an array of `MapCallbackReturn<N>` values.
     */
    dfs<C extends MapCallback<N>>(callback?: C, pattern?: DFSOrderPattern, beginRoot?: N | null, iterationType?: IterationType): ReturnType<C>[];
    /**
     * The bfs function performs a breadth-first search traversal on a binary tree, executing a callback
     * function on each node.
     * @param callback - The `callback` parameter is a function that will be called for each node in the
     * breadth-first search. It takes a node of type `N` as its argument and returns a value of type
     * `BFSCallbackReturn<N>`. The default value for this parameter is `this._defaultCallbackByKey
     * @param {N | null} beginRoot - The `beginRoot` parameter is the starting node for the breadth-first
     * search. It determines from which node the search will begin. If `beginRoot` is `null`, the search
     * will not be performed and an empty array will be returned.
     * @param iterationType - The `iterationType` parameter determines the type of iteration to be used
     * in the breadth-first search (BFS) algorithm. It can have two possible values:
     * @returns The function `bfs` returns an array of `BFSCallbackReturn<N>[]`.
     */
    bfs<C extends BFSCallback<N> = BFSCallback<N, BinaryTreeNodeKey>>(callback?: C, beginRoot?: N | null, iterationType?: IterationType): ReturnType<C>[];
    /**
     * The `listLevels` function takes a binary tree node and a callback function, and returns an array
     * of arrays representing the levels of the tree.
     * @param {C} callback - The `callback` parameter is a function that will be called on each node in
     * the tree. It takes a node as input and returns a value. The return type of the callback function
     * is determined by the generic type `C`.
     * @param {N | null} beginRoot - The `beginRoot` parameter represents the starting node of the binary tree
     * traversal. It can be any node in the binary tree. If no node is provided, the traversal will start
     * from the root node of the binary tree.
     * @param iterationType - The `iterationType` parameter determines whether the tree traversal is done
     * recursively or iteratively. It can have two possible values:
     * @returns The function `listLevels` returns an array of arrays, where each inner array represents a
     * level in a binary tree. Each inner array contains the return type of the provided callback
     * function `C` applied to the nodes at that level.
     */
    listLevels<C extends BFSCallback<N> = BFSCallback<N, BinaryTreeNodeKey>>(callback?: C, beginRoot?: N | null, iterationType?: IterationType): ReturnType<C>[][];
    /**
     * The function returns the predecessor node of a given node in a binary tree.
     * @param {N} node - The parameter "node" represents a node in a binary tree.
     * @returns The function `getPredecessor` returns the predecessor node of the given node `node`.
     */
    getPredecessor(node: N): N;
    /**
     * The `morris` function performs a depth-first traversal of a binary tree using the Morris traversal
     * algorithm and returns an array of values obtained by applying a callback function to each node.
     * @param callback - The `callback` parameter is a function that will be called on each node in the
     * tree. It takes a node of type `N` as input and returns a value of type `MapCallbackReturn<N>`. The
     * default value for this parameter is `this._defaultCallbackByKey`.
     * @param {DFSOrderPattern} [pattern=in] - The `pattern` parameter in the `morris` function
     * determines the order in which the nodes of a binary tree are traversed. It can have one of the
     * following values:
     * @param {N | null} beginRoot - The `beginRoot` parameter is the starting node for the Morris
     * traversal. It specifies the root node of the tree from which the traversal should begin. If
     * `beginRoot` is `null`, an empty array will be returned.
     * @returns The `morris` function returns an array of `MapCallbackReturn<N>` values.
     */
    morris<C extends MapCallback<N>>(callback?: C, pattern?: DFSOrderPattern, beginRoot?: N | null): ReturnType<C>[];
    /**
     * Swap the data of two nodes in the binary tree.
     * @param {N} srcNode - The source node to swap.
     * @param {N} destNode - The destination node to swap.
     * @returns {N} - The destination node after the swap.
     */
    protected _swap(srcNode: N, destNode: N): N;
    /**
     * Time complexity is O(n)
     * Space complexity of Iterative dfs equals to recursive dfs which is O(n) because of the stack
     * The Morris algorithm only modifies the tree's structure during traversal; once the traversal is complete,
     * the tree's structure should be restored to its original state to maintain the tree's integrity.
     * This is because the purpose of the Morris algorithm is to save space rather than permanently alter the tree's shape.
     */
    protected _defaultCallbackByKey: DefaultMapCallback<N>;
    /**
     * The function `_addTo` adds a new node to a binary tree if there is an available position.
     * @param {N | null} newNode - The `newNode` parameter represents the node that you want to add to
     * the binary tree. It can be either a node object or `null`.
     * @param {N} parent - The `parent` parameter represents the parent node to which the new node will
     * be added as a child.
     * @returns either the left or right child node of the parent node, depending on which child is
     * available for adding the new node. If a new node is added, the function also updates the size of
     * the binary tree. If neither the left nor right child is available, the function returns undefined.
     * If the parent node is null, the function also returns undefined.
     */
    protected _addTo(newNode: N | null, parent: N): N | null | undefined;
    /**
     * The function sets the root property of an object to a given value, and if the value is not null,
     * it also sets the parent property of the value to undefined.
     * @param {N | null} v - The parameter `v` is of type `N | null`, which means it can either be of
     * type `N` or `null`.
     */
    protected _setRoot(v: N | null): void;
    /**
     * The function sets the value of the protected property "_size" to the given number.
     * @param {number} v - The parameter "v" is a number that represents the size value that we want to
     * set.
     */
    protected _setSize(v: number): void;
}
